﻿using System;
using System.Text;
using System.Text.RegularExpressions;
using System.Globalization;
using System.IO;



namespace Extentions
{
    public static class StringExtentions
    {
        /// <summary>
        /// 文字列を末尾に追加します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="values">追加する文字列</param>
        /// <returns>末尾に文字列を追加した結果の文字列を返します。</returns>
        public static string Append(this string @this, params string[] values)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(@this);
            foreach (string value in values)
            {
                sb.Append(value);
            }
            return sb.ToString();
        }

        /// <summary>
        /// カルチャに依存した比較、または大文字と小文字を区別しない比較などの序数による比較を指定できる Contains。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="value">検索する文字列</param>
        /// <param name="comparison">カルチャ、大文字と小文字の区別、および並べ替え規則を指定する StringComparison</param>
        /// <returns>一致する場合に true を返します。</returns>
        public static bool Contains(this string @this, string value, StringComparison comparison)
        {
            return @this.IndexOf(value, comparison) >= 0;
        }

        /// <summary>
        /// 部分一致・前方一致・後方一致するかどうかを判定します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="value">検索する文字列</param>
        /// <param name="likeStyle">一致スタイル</param>
        /// <returns>一致する場合に true を返します。</returns>
        public static bool Contains(this string @this, string value, ContainsStyle containsStyle = ContainsStyle.Normal)
        {
            bool isMatch = false;
            switch (containsStyle)
            {
                case ContainsStyle.Normal:
                    // 部分一致
                    isMatch = @this.Contains(value);
                    break;
                case ContainsStyle.Forward:
                    // 前方一致
                    isMatch = @this.StartsWith(value);
                    break;
                case ContainsStyle.Backward:
                    // 後方一致
                    isMatch = @this.EndsWith(value);
                    break;
            }
            return isMatch;
        }

        /// <summary>
        /// Contains の一致スタイル
        /// </summary>
        public enum ContainsStyle
        {
            /// <summary>
            /// 部分一致
            /// </summary>
            Normal = 0,
            /// <summary>
            /// 前方一致
            /// </summary>
            Forward,
            /// <summary>
            /// 後方一致
            /// </summary>
            Backward
        }

        /// <summary>
        /// 指定した複数の文字列のすべてを含むかどうかを判定します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="values">検索する文字列</param>
        /// <returns>指定した複数の文字列のすべてを含めば true を返します。</returns>
        public static bool ContainsAll(this string @this, params string[] values)
        {
            foreach (string value in values)
            {
                if (!@this.Contains(value))
                {
                    return false;
                }
            }
            return true;
        }
        
        /// <summary>
        /// 指定した複数の文字列のいずれかを含むかどうかを判定します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="values">検索する文字列</param>
        /// <returns>指定した複数の文字列のいずれかを含めば true を返します。</returns>
        public static bool ContainsAny(this string @this, params string[] values)
        {
            foreach (string value in values)
            {
                if (@this.Contains(value))
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// 指定した文字列が最初に一致した位置より後の文字列を切り取ります。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="value">検索する値</param>
        /// <returns>切り取った文字列を返します。</returns>
        public static string CutOutAfter(this string @this, string value)
        {
            if (@this.IndexOf(value) == -1)
            {
                return "";
            }
            return @this.Substring(@this.IndexOf(value) + value.Length);
        }

        /// <summary>
        /// 指定した文字列が最初に一致した位置より前の文字列を切り取ります。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="value">検索する値</param>
        /// <returns>切り取った文字列を返します。</returns>
        public static string CutOutBefore(this string @this, string value)
        {
            if (@this.IndexOf(value) == -1)
            {
                return "";
            }
            return @this.Substring(0, @this.IndexOf(value));
        }

        /// <summary>
        /// 指定した２つの文字列の間の文字列を切り取ります。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="beforeValue">検索する前の文字列</param>
        /// <param name="afterValue">検索する後の文字列</param>
        /// <returns>切り取った文字列を返します。</returns>
        public static string CutOutBetween(this string @this, string beforeValue, string afterValue)
        {
            int beforeStartIndex = @this.IndexOf(beforeValue);
            int startIndex = beforeStartIndex + beforeValue.Length;
            int afterStartIndex = @this.IndexOf(afterValue, startIndex);

            if (beforeStartIndex == -1 || afterStartIndex == -1)
            {
                return "";
            }

            return @this.Substring(startIndex, afterStartIndex - startIndex);
        }

        /// <summary>
        /// 文字列から指定した文字列 (複数指定可能) を削除します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="removeValues">削除する文字列の配列</param>
        /// <returns>削除した結果の文字列を返します。</returns>
        public static string Remove(this string @this, params string[] removeValues)
        {
            string resultValue = @this;
            foreach (string removeValue in removeValues)
            {
                resultValue = resultValue.Replace(removeValue, string.Empty);
            }
            return resultValue;
        }

        /// <summary>
        /// 文字列を名前を付けて保存します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="fileName">ファイル名</param>
        /// <param name="append">ファイルに追加するかどうか (true: 追加, false: 上書き)</param>
        public static void SaveAs(this string @this, string fileName, bool append = false, Encoding encoding = null)
        {
            if (null == encoding)
            {
                encoding = Encoding.Default;
            }

            using (TextWriter tw = new StreamWriter(fileName, append, encoding))
            {
                tw.Write(@this);
            }
        }

        /// <summary>
        /// 指定した区切り文字で分割した値を配列に格納します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="separator">区切り文字</param>
        /// <param name="options">配列に空文字列を含む要素を格納するかどうかを示す値</param>
        /// <returns>区切り文字列で分割して配列に格納した結果を返します。</returns>
        public static string[] Split(this string @this, string separator, StringSplitOptions options = StringSplitOptions.None)
        {
            return @this.Split(new[] { separator }, options);
        }

        /// <summary>
        /// 分割する回数と区切り文字を指定して分割した値を配列に格納します。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <param name="separator">区切り文字列</param>
        /// <param name="count">区切り文字で分割する回数</param>
        /// <param name="options">配列に空文字列を含む要素を格納するかどうかを示す値</param>
        /// <returns>区切り文字列で分割して配列に格納した結果を返します。</returns>
        public static string[] Split(this string @this, string separator, int count, StringSplitOptions options = StringSplitOptions.None)
        {
            return @this.Split(new[] { separator }, count, options);
        }


        /// <summary>
        /// 連続した空白を単一の空白に置き換え。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <returns>単一スペースに置き換えられた文字列を返します。</returns>
        public static string RemoveMultipleSpace(this string @this)
        {
            return System.Text.RegularExpressions.Regex.Replace(@this, @"\s+", " ");
        }

        /// <summary>
        /// 全角空白、半角空白、タブスペースをすべて削除。
        /// </summary>
        /// <param name="this">対象の文字列</param>
        /// <returns>スペースを削除された文字列を返します。</returns>
        public static string RemoveAllSpace(this string @this)
        {
            return System.Text.RegularExpressions.Regex.Replace(@this, @"\s", "");
        }

    }

}